unit SpinMain;

interface

uses
  Windows, SysUtils, Classes, Controls, Forms,
  Dialogs, JvLED, StdCtrls, ExtCtrls,
  spinkit, JvExControls, JvComponent;

type
  TSpinMainForm = class(TForm)
    JvLED1: TJvLED;
    JvLED2: TJvLED;
    JvLED3: TJvLED;
    JvLED4: TJvLED;
    JvLED5: TJvLED;
    JvLED6: TJvLED;
    JvLED7: TJvLED;
    Spin1: TLabel;
    Spin2: TLabel;
    Spin3: TLabel;
    Spin4: TLabel;
    Sum1: TLabel;
    Sum2: TLabel;
    Sum3: TLabel;
    Sum4: TLabel;
    Spin5: TLabel;
    Sum5: TLabel;
    Spin6: TLabel;
    Sum6: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Edit1: TEdit;
    Timer1: TTimer;
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  public
    LEDs: array [1..7] of TJvLED;
    Labels: array [1..6] of TLabel;
    Spins: array [1..6] of TLabel;
    Sums: array [1..6] of TLabel;
    SumValues: array [1..6] of Integer;
    SpinDevice: SPINKIT_HANDLE;
    SpinPID: ULONG;
    CallbackMsg: ULONG;
    Accumulator: Integer;
    procedure AfterConstruction; override;
  end;

var
  SpinMainForm: TSpinMainForm;

implementation

{$R *.dfm}

procedure TSpinMainForm.AfterConstruction;
var
  I: Integer;
  Buffer: array [0..9] of WideChar;
  Name: string;
begin
  // initialize the control arrays with the controls
  for I := Low(LEDs) to High(LEDs) do
    LEDs[I] := TJvLED(FindComponent(Format('JvLED%u', [I])));
  for I := Low(Labels) to High(Labels) do
    Labels[I] := TLabel(FindComponent(Format('Label%u', [I])));
  for I := Low(Spins) to High(Spins) do
    Spins[I] := TLabel(FindComponent(Format('Spin%u', [I])));
  for I := Low(Sums) to High(Sums) do
    Sums[I] := TLabel(FindComponent(Format('Sum%u', [I])));
  for I := Low(SumValues) to High(SumValues) do
    SumValues[I] := 0;
  Accumulator := 0;

  // open up the first (and only) SpinWarrior
  SpinDevice := SpinKitOpenDevice;

  // enable/disable controls depending on SpinWarrior type
  for I := Low(Spins) to High(Spins) do
  begin
    Labels[I].Enabled := Assigned(SpinDevice);
    Spins[I].Enabled := Assigned(SpinDevice);
    Sums[I].Enabled := Assigned(SpinDevice);
  end;

  if Assigned(SpinDevice) then
    SpinPID := SpinKitGetProductId(SpinDevice)
  else
    SpinPID := 0;

  case SpinPID of
    SPINKIT_PRODUCT_ID24R4:
      begin
        for I := 5 to High(Spins) do
        begin
          Labels[I].Enabled := False;
          Spins[I].Enabled := False;
          Sums[I].Enabled := False;
        end;
        Name := 'SpinWarrior 24R4';
      end;
    SPINKIT_PRODUCT_ID24R6:
      begin
        for I := 4 to High(LEDs) do
          LEDs[I].Visible := False;
        Name := 'SpinWarrior 24R6';
      end;
    SPINKIT_PRODUCT_ID24A3:
      begin
        for I := Low(Spins) to High(Spins) do
          Sums[I].Enabled := False;
        for I := 4 to High(Spins) do
        begin
          Labels[I].Enabled := False;
          Spins[I].Enabled := False;
        end;
        for I := 7 to High(LEDs) do
          LEDs[I].Visible := False;
        Name := 'SpinWarrior 24A3';
      end;
  end;

  if Assigned(SpinDevice) then
  begin
    // show the serial number of the SpinWarrior
    SpinKitGetSerialNumber(SpinDevice, Buffer);
    Caption := Name + ' - ' + Buffer;
    Timer1.Enabled := True;
  end;
end;

procedure TSpinMainForm.FormDestroy(Sender: TObject);
begin
  SpinKitCloseDevice(SpinDevice);
end;

procedure TSpinMainForm.Timer1Timer(Sender: TObject);
var
  I: Integer;
  SpinData: SPINKIT_DATA;
begin
  while SpinKitReadNonBlocking(SpinDevice, SpinData) do
  begin
    // show the button status
    for I := Low(LEDs) to High(LEDs) do
      LEDs[I].Status := SpinData.Buttons[I-1];

    // show the spin status
    for I := Low(Spins) to High(Spins) do
    begin
      SumValues[I] := SumValues[I] + SpinData.Spins[I-1];
      if Spins[I].Enabled then
        Spins[I].Caption := Format('%d', [SpinData.Spins[I-1]])
      else
        Spins[I].Caption := '-';
      if Sums[I].Enabled then
        Sums[I].Caption := Format('%d', [SumValues[I]])
      else
        Sums[I].Caption := '-';
    end;

    // the electromechanical spins send about four pulses per click
    // so we keep an accumulator to divide it by four
    if SpinPID <> SPINKIT_PRODUCT_ID24A3 then
    begin
      Accumulator := Accumulator + SpinData.Spins[2];
      while Accumulator >= 4 do
      begin
        CMSendSpinChar(1);
        Dec(Accumulator, 4);
      end;
      while Accumulator <= -4 do
      begin
        CMSendSpinChar(-1);
        Inc(Accumulator, 4);
      end;
    end;
  end;
end;

end.
